#================================================
# 2003  Jeroen C. van Nieuwenhuizen
# $Revision: 1.5 $
# $Date: 2003/09/13 14:12:15 $
# $Author: jnieuwen $
# $Name:  $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the license, or 
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#================================================

=head1 DESCRIPTION

The wormdb object provides a record based interface to the wormwarner database.

=head1 FUNCTIONS

=cut

package wormdb ;

use strict ; # Make the code better :-)
use POSIX qw(strftime);
use GDBM_File ;
use Fcntl;

#================================================

=over 4

=item B<new()>

Makes a database record object.

B<Parameters>: I<database_location>, I<key_ip>

B<Return values>: A reference to the newly created object.

=cut

sub new {
	my $class = shift ;
	my $self = {} ;

	$self->{dbfile} = shift ;
	$self->{ip} = shift ;
	$self->{first_scan} = -1 ;
	$self->{last_scan} = -1 ;
	$self->{last_warning} = -1 ;
	$self->{total_incidents} = 0 ;
	$self->{total_isp_warnings} = 0 ;
	$self->{nimda} = 0 ;
	$self->{coderedI} = 0 ;
	$self->{coderedII} = 0 ;
	$self->{sadmin} = 0 ;
	$self->{slapper} = 0 ;
	$self->{attacks} = 0 ;
	$self->{incidents_since_last} = 0 ;
	$self->{address} = "" ;
	$self->{active} = 0 ;
	$self->{block_until} = 0 ;

	bless $self ;
	return $self ;
}
#================================================

=item B<write()>

writes the object (record) to the database.

B<Return values>: 1 on success. 0 on failure

=cut

sub write {
	my $self = shift ;
	my %database ;
	my $entry = join ("::",
		$self->{first_scan},
		$self->{last_scan},
		$self->{last_warning},
		$self->{total_incidents},
		$self->{total_isp_warnings},
		$self->{nimda},
		$self->{coderedI},
		$self->{coderedII},
		$self->{sadmin},
		$self->{slapper},
		$self->{attacks},
		$self->{incidents_since_last},
		$self->{address},
		$self->{active},
		$self->{block_until}
	) ;

	tie (%database, 'GDBM_File', $self->{dbfile},&GDBM_WRCREAT, 0644) or return 0 ;
	$database{$self->{ip}} = $entry ;
	untie %database ;

	return 1 ;
}
#================================================

=item B<read()>

Reads a record from the database. If the database does not contain
information for the key (ip) then the record will contain the default
values.

B<Return values>: 0 if the database could not be found or is corrupted.
1 otherwise.

=cut

sub read {
	my $self = shift ;
	my %database ;
	my $entry ;

	tie (%database,'GDBM_File',$self->{dbfile},&GDBM_READER, 0644) or return 0 ;
	$entry = $database{$self->{ip}} ;
	untie %database ;
	(
		$self->{first_scan},
		$self->{last_scan},
		$self->{last_warning},
		$self->{total_incidents},
		$self->{total_isp_warnings},
		$self->{nimda},
		$self->{coderedI},
		$self->{coderedII},
		$self->{sadmin},
		$self->{slapper},
		$self->{attacks},
		$self->{incidents_since_last},
		$self->{address},
		$self->{active},
		$self->{block_until}
	) = split(/::/,$entry) if (defined $entry) ;

	return 1 ;
}
#================================================

=item B<last_warning_now()>

Sets the time of the last warning to the current time.
The incidents since the last warning counter is reset to 0.

=cut

sub last_warning_now {
	my $self = shift ;
	$self->{last_warning} = time() ;
	$self->{incidents_since_last} = 0 ;
}
#================================================

=item B<last_isp_warning_now()>

Sets the time of the last warning to the current time.
The incidents since the last warning counter is reset to 0.
The number of warnings send to the ISP counter is incremented with 1.

=cut

sub last_isp_warning_now {
	my $self = shift ;
	$self->{last_warning} = time() ;
	$self->{incidents_since_last} = 0 ;
	$self->{total_isp_warnings}++ ;
}
#================================================

=item B<last_scan_now()>

Sets the time of the last scan to the current time.  If the time of the first
scan is unspecified it is also set to the current time.

=cut

sub last_scan_now {
	my $self = shift ;
	$self->{last_scan} = time() ;
	$self->{first_scan} = $self->{last_scan} if ($self->{first_scan} == -1) ;
}
#================================================

=item B<nimda()>

Updates the record for the occurence of a nimda scan.

=cut

sub nimda {
	my $self = shift ;
	$self->{nimda}++ ;
	$self->{total_incidents}++ ;
	$self->{incidents_since_last}++ ;
	$self->last_scan_now();
}
#================================================

=item B<codered1()>

Updates the record for the occurence of a CodeRed I scan.

=cut

sub codered1 {
	my $self = shift ;
	$self->{coderedI}++ ;
	$self->{total_incidents}++ ;
	$self->{incidents_since_last}++ ;
	$self->last_scan_now();
}
#================================================

=item B<codered2()>

Updates the record for the occurence of a CodeRed II scan.

=cut

sub codered2 {
	my $self = shift ;
	$self->{coderedII}++ ;
	$self->{total_incidents}++ ;
	$self->{incidents_since_last}++ ;
	$self->last_scan_now();
}
#================================================

=item B<sadmin()>

Updates the record for the occurence of a sadmin scan.

=cut

sub sadmin {
	my $self = shift ;
	$self->{sadmin}++ ;
	$self->{total_incidents}++ ;
	$self->{incidents_since_last}++ ;
	$self->last_scan_now();
}
#================================================

=item B<slapper()>

Updates the record for the occurence of a slapper or scalper scan.

=cut

sub slapper {
	my $self = shift ;
	$self->{slapper}++ ;
	$self->{total_incidents}++ ;
	$self->{incidents_since_last}++ ;
	$self->last_scan_now();
}
#================================================

=item B<attack()>

Updates the record for the occurence of an attack.

=cut

sub attack {
	my $self = shift ;
	$self->{attacks}++ ;
	$self->{total_incidents}++ ;
	$self->{incidents_since_last}++ ;
	$self->last_scan_now();
}
#================================================

=item B<is_pending()>

Checks whether the last warning was send less then I<days> ago and that the ISP was never warned before.

B<Parameters>: I<days>

B<Return values>: 1 if true, 0 if false.

=cut

sub is_pending {
	my $self = shift ;
	my $days = shift ;
	my $seconds = 3600 * 24 * $days ;
	return 1 if ($self->{total_isp_warnings} == 0 && (time() - $self->{last_warning} < $seconds)) ;
	return 0 ;
}
#================================================

=item B<isp_pending()>

Checks whether the last warning was send less then I<days> ago.

B<Parameters>: I<days>

B<Return values>: 1 if true, 0 if false.

=cut

sub isp_pending {
	my $self = shift ;
	my $days = shift ;
	my $seconds = 3600 * 24 * $days ;
	return 1 if ((time() - $self->{last_warning} < $seconds)) ;
	return 0 ;
}
#================================================

=item B<isp_warned()>

Checks whether a warning was send to the ISP for record.

B<Return values>: 1 if true, 0 if false.

=cut

sub isp_warned {
	my $self = shift ;
	return 1 if ($self->{total_isp_warnings} > 0) ;
	return 0 ;
}
#================================================

=item B<was_warned()>

Checks whether a warning was send for this record.

B<Return values>: 1 if true, 0 if false.

=cut

sub was_warned {
	my $self = shift ;
	return 0 if ($self->{total_incidents} == 0) ;
	return 1 ;
}
#================================================

=item B<set_block_until()>

Sets the ending of the block for the current host to
now + I<days>.

B<Parameters>: I<days>

=cut

sub set_block_until {
	my $self = shift ;
	my $days = shift ;
	my $seconds = 3600 * 24 * $days ;
	$self->{block_until} = time() + $seconds ;
}
#================================================

=item B<set_active()>

Sets this item to active.

=cut

sub set_active {
	my $self = shift ;
	$self->{active} = 1 ;
}
#================================================

=item B<reset_active()>

Sets this item to inactive.

=cut

sub reset_active {
	my $self = shift ;
	$self->{active} = 0 ;
}
#================================================

=item B<return_all_expired()>

Returns all keys in the database for which the time specified
by block_until is smaller then the current time.

B<Return value>: an array containing all expired items.

=cut

sub return_all_expired {
	my $self = shift ;
	my %entry ;
	my %database ;
	my @keys ;

	tie (%database,'GDBM_File',$self->{dbfile},&GDBM_READER, 0644) or return 0 ;
	while (my ($k,$v) = each %database) {
		if (defined $v) {
			(
				$entry{first_scan},
				$entry{last_scan},
				$entry{last_warning},
				$entry{total_incidents},
				$entry{total_isp_warnings},
				$entry{nimda},
				$entry{coderedI},
				$entry{coderedII},
				$entry{sadmin},
				$entry{slapper},
				$entry{attacks},
				$entry{incidents_since_last},
				$entry{address},
				$entry{active},
				$entry{block_until}
			) = split(/::/,$v) ;
			if ($entry{block_until} < time() && $entry{active} == 1) {
				push (@keys,$k) ;
			}
		}
	}
	untie %database ;
	return @keys ;
}
#================================================

=back

=head1 AUTHOR

Written by Jeroen van Nieuwenhuizen.

=head1 VERSION

$Revision: 1.5 $ $Date: 2003/09/13 14:12:15 $

$Source: /home/jnieuwen/cvsrepos/squid/wormdb.pm,v $

=head1 REPORTING BUGS

Report bugs to <jnieuwen@softhome.net>

=head1 COPYRIGHT

Copyright (C) 2003 by Jeroen van Nieuwenhuizen. Distributed
under the GNU GPL. See the file "F<LICENSE>", supplied with the distribution
for additional information.

=cut

1;
